//------------------------------------------------------------------------------
// CERN BE-CO-HT
// Mock Turtle
// https://gitlab.cern.ch/coht/mockturtle
//------------------------------------------------------------------------------
//
// unit name:   MockTurtleDriver
//
// description: A SystemVerilog Class to provide an abstraction of the complete
// MockTurtle core.
//
//------------------------------------------------------------------------------
// Copyright (c) 2018 CERN
//------------------------------------------------------------------------------
// GNU LESSER GENERAL PUBLIC LICENSE
//------------------------------------------------------------------------------
// This source file is free software; you can redistribute it and/or modify it
// under the terms of the GNU Lesser General Public License as published by the
// Free Software Foundation; either version 2.1 of the License, or (at your
// option) any later version. This source is distributed in the hope that it
// will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU Lesser General Public License for more details. You should have
// received a copy of the GNU Lesser General Public License along with this
// source; if not, download it from http://www.gnu.org/licenses/lgpl-2.1.html
//------------------------------------------------------------------------------

`ifndef __MOCK_TURTLE_DRIVER_INCLUDED
 `define __MOCK_TURTLE_DRIVER_INCLUDED

 `include "simdrv_defs.svh"
 `include "mt_cpu_csr_driver.svh"
 `include "mt_mqueue_host_driver.svh"
 `include "mt_config_rom_driver.svh"

class MockTurtleDriver;
   protected CBusAccessor m_acc;
   protected uint64_t m_base;
   protected MTCPUControl m_csr;
   protected MQueueHost m_hmq;
   protected MTConfigRom m_rom;

   function  new ( CBusAccessor acc, uint64_t base );
      m_acc = acc;
      m_base = base;
   endfunction // new

   task init();
      m_csr = new ( m_acc, m_base + 'hc000 );
      m_hmq = new ( m_acc, m_base + 'h0000 );
      m_rom = new (m_acc, m_base + 'he000, 256 );
      m_csr.init();
      m_hmq.init();
      m_rom.init();
   endtask // init

   task load_firmware ( int core, string filename );
      m_csr.load_firmware ( core, filename );
   endtask // load_firmware

   task reset_core ( int core, int reset );
      m_csr.reset_core ( core, reset );
   endtask // reset_core

   task cfg_rom_display();
      m_rom.pretty_print();
   endtask // cfg_rom_display

   // polls HMQ and Debug UART
   task update();
      m_csr.update();
      m_hmq.update();
   endtask // update

endclass // MockTurtleDriver

`endif //  `ifndef __MOCK_TURTLE_DRIVER_INCLUDED
