//------------------------------------------------------------------------------
// CERN BE-CO-HT
// Mock Turtle
// https://gitlab.cern.ch/coht/mockturtle
//------------------------------------------------------------------------------
//
// unit name:   MTConfigRom
//
// description: A ROM to hold the various configuration values of the current
// MockTurtle implementation.
//
//------------------------------------------------------------------------------
// Copyright (c) 2018 CERN
//------------------------------------------------------------------------------
// GNU LESSER GENERAL PUBLIC LICENSE
//------------------------------------------------------------------------------
// This source file is free software; you can redistribute it and/or modify it
// under the terms of the GNU Lesser General Public License as published by the
// Free Software Foundation; either version 2.1 of the License, or (at your
// option) any later version. This source is distributed in the hope that it
// will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU Lesser General Public License for more details. You should have
// received a copy of the GNU Lesser General Public License along with this
// source; if not, download it from http://www.gnu.org/licenses/lgpl-2.1.html
//------------------------------------------------------------------------------

`ifndef __MOCK_TURTLE_CONFIG_ROM_INCLUDED
 `define __MOCK_TURTLE_CONFIG_ROM_INCLUDED

 `include "simdrv_defs.svh"

class MTConfigRom;
   protected CBusAccessor m_acc;
   protected uint64_t     m_base;
   protected uint32_t     m_size;
   protected uint32_t     m_data [];

   function new ( CBusAccessor acc, uint64_t base, uint64_t size );
      m_acc  = acc;
      m_base = base;
      m_size = size;
      m_data = new[m_size];
   endfunction // new

   task _readl (  uint32_t r, ref uint32_t v );
      uint64_t tmp;
      m_acc.read ( m_base + r, tmp );
      v= tmp;
   endtask // _readl

   task init ( );
      int i;

      for ( i = 0; i < m_size; i+=4 )
	begin
	   _readl ( i, m_data[i] );
	end
   endtask // init

   function uint32_t read ( uint32_t pos );
      return m_data[4 * pos];
   endfunction // read

   task pretty_print ( );
      $display ( "Configuration ROM contents:" );
      $display ( "  Magic Value: %s",  read(0) );
      $display ( "  MT revision: %0d", read(1) );

   endtask // pretty_print

endclass // MockTurtleConfigRom

`endif // `ifndef __MOCK_TURTLE_CONFIG_ROM_DRIVER_INCLUDED
