`include "mock_turtle_driver.svh"
`include "vhd_wishbone_master.svh"

`timescale 1ns/1ps

module main;

   reg rst_n = 0;
   reg clk_sys = 0;

   always #8ns clk_sys <= ~clk_sys;

   initial begin
      repeat(20) @(posedge clk_sys);
      rst_n = 1;
   end

   wire host_irq;

   IVHDWishboneMaster Host ( clk_sys, rst_n );

   mock_turtle_core #
     (
      .g_double_core_clock(1'b0)
      )
   DUT (
	.clk_i   (clk_sys),
	.rst_n_i (rst_n),
	.host_slave_i (Host.master.out),
	.host_slave_o (Host.master.in),
	.host_irq_o(host_irq)
	);

   initial begin
      automatic CWishboneAccessor acc;
      automatic MockTurtleDriver drv;
      automatic MTCPUControl csr;

      #10us;

      acc = Host.get_accessor();
      acc.set_mode (PIPELINED);

      drv = new(acc, 'h20000);
      drv.init();

      csr = drv.CPUs();

      // enable debug message interrupts for CPU0
      csr.debug_int_enable(0, 1);

      csr.load_firmware(0, "../sw/hello/hello.ram.urv");
      csr.reset_core(0, 0);

      forever begin
	 drv.update();
	 #1us;
      end

   end // initial begin

endmodule
