/*
 * Copyright (C) 2016 CERN (www.cern.ch)
 * Author: Federico Vaga <federico.vaga@cern.ch>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <mockturtle-framework.h>

/**
 * The function can be used to send debug information to the host
 * trought the HMQ
 * @param[in] mq_in MQ index within mq declaration in rt_application
 * @param[in] n_values number of variadic arguments
 * @return 0 on success. -1 on error
 */
#ifdef MOCKTURTLE_FRAMEWORK_DEBUG_ENABLE
int rt_send_debug(int mq_in, int n_values, ...)
{
	va_list ap;
	struct trtl_msg out_buf;
	uint32_t *buf;
	struct trtl_proto_header hdr = {
		.rt_app_id = _app->version.rt_id,
		.msg_id = TRTL_MSG_ID_DBG,
		.slot_io = _app->mq[mq_in].index & 0xF,
		.seq = 0,
		.len = n_values,
		.flags = 0x0,
		.trans = 0x0,
		.time = 0x0,
	};
	int i;

	out_buf = rt_mq_claim_out(&hdr);
	buf = (uint32_t *)rt_proto_payload_get(out_buf.data);

	va_start(ap, n_values);
	for (i = 0; i < hdr.len;++i)
		buf[i] = va_arg(ap, uint32_t);
	va_end(ap);

	rt_proto_header_set((void *) out_buf.data, &hdr);
	rt_mq_msg_send(&out_buf);

	return 0;
}
#else
int rt_send_debug(int mq_in, int n_values, ...)
{
	return 0;
}
#endif



/**
 * It prints on the serial interface the given message header
 * @param[in] h message header to print
 */
#ifdef MOCKTURTLE_FRAMEWORK_DEBUG_ENABLE
void rt_print_header(struct trtl_proto_header *h)
{
	pr_debug("    app_id 0x%x | msg_id %d | slot_io 0x%x | seq %"PRIu32"d\n\r",
		  h->rt_app_id, h->msg_id, h->slot_io, h->seq);
	delay(1000);
	pr_debug("    len %d | flags 0x%x | trans 0x%x | time %"PRIu32"d\n\r",
		  h->len, h->flags, h->trans, h->time);
	delay(1000);
}
#else
void rt_print_header(struct trtl_proto_header *h)
{
	return;
}
#endif


/**
 * It prints on the serial interface a buffer
 * @param[in] d buffer data
 * @param[in] count number of 32bit words to show
 */
#ifdef MOCKTURTLE_FRAMEWORK_DEBUG_ENABLE
void rt_print_data(uint32_t *d, unsigned int count)
{
	int i;

	for (i = 0; i < count; i++) {
		pr_debug("%s: data[%d] = 0x%"PRIx32"\n\r", __func__, i , d[i]);
		delay(1000);
	}
}
#else
void rt_print_data(uint32_t *d, unsigned int count)
{
	return ;
}
#endif
